<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="kompare_shell" version="9">
<MenuBar>
  <Menu noMerge="1" name="file"><text>&amp;File</text>
    <Action name="file_open"/>
    <Action name="file_compare_files"/>
    <Action name="file_blend_url"/>
    <Separator/>
    <Merge/>
    <Separator/>
    <Action name="file_quit"/>
  </Menu>
  <Merge/>
  <Menu noMerge="1" name="settings"><text>&amp;Settings</text>
    <Merge name="StandardToolBarMenuHandler" />
    <Action name="options_show_toolbar"/>
    <Action name="options_show_statusbar"/>
    <Action name="options_show_text_view"/>
    <Merge name="show_merge"/>
    <Separator/>
    <Action name="options_configure_keybinding"/>
    <Action name="options_configure_toolbars"/>
    <Merge name="configure_merge"/>
    <Merge/>
  </Menu>
</MenuBar>
<ToolBar noMerge="1" name="mainToolBar"><text>Main Toolbar</text>
  <Action name="file_compare_files"/>
  <Separator/>
  <Merge/>
  <Action name="help"/>
</ToolBar>
</gui>
